@@ -1,4 +1,4 @@
-# Copyright (C) 2020-2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # Contains the main sequencing of the compiler.
 import time
@@ -64,6 +65,7 @@ class CompilerOptions:
         show_cpu_operations=False,
         tensor_allocator=TensorAllocator.Greedy,
         timing=False,
+        force_symmetric_int_weights=False,
         output_dir="outputs",
         cpu_tensor_alignment=Tensor.AllocationQuantum,
         hillclimb_max_iterations=None,
@@ -83,6 +85,7 @@ class CompilerOptions:
         self.show_cpu_operations = show_cpu_operations
         self.tensor_allocator = tensor_allocator
         self.timing = timing
+        self.force_symmetric_int_weights = force_symmetric_int_weights
         self.output_dir = output_dir
         self.cpu_tensor_alignment = cpu_tensor_alignment
         self.hillclimb_max_iterations = hillclimb_max_iterations
@@ -133,7 +136,7 @@ def next_sram_factor(alloc_results):
 
 
 def _record_operator(op, arch):
-    if op.type != Op.Const:
+    if op.type not in (Op.Const, Op.Placeholder):
         DebugDatabase.add_source(op)
 
 
@@ -149,14 +152,16 @@ def _check_schedule(nng, arch, scheduler_options):
             )
 
 
-def compiler_driver(nng, arch, options, scheduler_options, network_type):
+def compiler_driver(nng, arch, options, scheduler_options, network_type, output_basename):
     assert verify_graph_health(nng)
 
     # Pre-optimisation operator tracking
     for sg in nng.subgraphs:
         visit_graph_post_order(sg.output_tensors, arch, [], [_record_operator])
 
-    nng = graph_optimiser.optimise_graph(nng, arch, network_type, options.verbose_graph)
+    nng = graph_optimiser.optimise_graph(
+        nng, arch, network_type, options.verbose_graph, options.force_symmetric_int_weights
+    )
     assert verify_graph_health(nng)
 
     if options.verbose_quantization:
@@ -233,7 +238,10 @@ def compiler_driver(nng, arch, options, scheduler_options, network_type):
             sg, arch, scratch_tens, scratch_fast_tens, flash_tens
         )
 
-    npu_serialisation.rewrite_npu_call_ops(root_sg, arch)
+    # Create list of CPU subgraphs with same order as the list of all subgraphs
+    cpu_subgraphs = [sg for sg in nng.subgraphs if sg.placement == PassPlacement.Cpu]
+    for sg in cpu_subgraphs:
+        npu_serialisation.rewrite_npu_call_ops(sg, arch)
 
     # Set Scratch and Fast_scratch Tensor size
     if scratch_tens is not None:
@@ -254,4 +262,6 @@ def compiler_driver(nng, arch, options, scheduler_options, network_type):
         cpu_tensor_alignment=options.cpu_tensor_alignment,
     )
 
-    npu_performance.calc_new_performance_for_network(nng, arch, network_type, options.verbose_performance)
+    npu_performance.calc_new_performance_for_network(
+        nng, arch, network_type, options.verbose_performance, output_basename
+    )
