@@ -1,4 +1,4 @@
-# Copyright (C) 2020 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2021, 2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # Defines the basic numeric type classes for tensors.
 import enum
@@ -109,7 +110,7 @@ class DataType:
             BaseType.Complex: "c",
         }
         assert self.type in numpy_dtype_code, f"Failed to interpret {self} as a numpy dtype"
-        return np.dtype(numpy_dtype_code[self.type] + str(self.size_in_bytes()))
+        return np.dtype(numpy_dtype_code[self.type] + str(self.size_in_bytes())).type
 
     stem_name = {
         BaseType.UnsignedInt: ("uint%s", True),
