@@ -1,4 +1,4 @@
-# Copyright (C) 2020-2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2022 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # Generate a high-level command stream from a schedule
 from .high_level_command_stream import Box
@@ -74,6 +75,8 @@ def generate_high_level_commands_for_sched_op(sched_op, schedule):
         _,
         _,
     ) = parent_op.get_ifm_ifm2_weights_biases_ofm()
+    if sched_op.reversed_operands:
+        ifm2_tensor, ifm_tensor = ifm_tensor, ifm2_tensor
     ifm = sched_op.ifm
     ifm2 = sched_op.ifm2
     ofm_shape = sched_op.ofm.shape
@@ -204,12 +207,13 @@ def generate_high_level_commands_for_sched_op(sched_op, schedule):
                 if op_info.npu_weights_tensor:
                     weight_box = Box([0, 0, 0, start_channel], [1, 1, 1, end_channel])
 
-                    if op_info.buffered_weight_tensors and is_first_h_stripe:
+                    if op_info.buffered_weight_tensors:
                         idx = depth_idx % len(op_info.buffered_weight_tensors)
-                        yield from dma_if_necessary(
-                            sched_op.parent_ps, weight_box, op_info.buffered_weight_tensors[idx]
-                        )
                         weight_tensor = op_info.buffered_weight_tensors[idx]
+                        if is_first_h_stripe:
+                            yield from dma_if_necessary(
+                                sched_op.parent_ps, weight_box, op_info.buffered_weight_tensors[idx]
+                            )
                 else:
                     weight_box = None
 
@@ -236,4 +240,5 @@ def generate_high_level_commands_for_sched_op(sched_op, schedule):
                     ifm2_box=ifm2_box,
                     pad_top=pad_top,
                     pad_bottom=pad_bottom,
+                    reversed_operands=sched_op.reversed_operands,
                 )
