@@ -1,4 +1,4 @@
-# Copyright (C) 2020-2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # Neural network graph classes and enums.
 # Pass - A packed pass containing one or more Operations.
@@ -148,7 +149,11 @@ class Subgraph:
     def __init__(self, name="<unnamed>", placement=PassPlacement.Cpu):
         self.output_tensors = []
         self.input_tensors = []
-        self.original_inputs = []  # Preserve the original input order
+        # Preserve the original input order
+        self.original_inputs = []
+        # Attach virtual outputs to resource variables op
+        # in order to be able to traverse the graph correctly
+        self.virtual_outputs = []
         self.passes = []
         self.cascaded_passes = []
         self.name = name
@@ -196,7 +201,8 @@ class Subgraph:
 
             if op.type in (Op.Placeholder, Op.SubgraphInput):
                 assert len(op.outputs) == 1
-                self.input_tensors.append(op.outputs[0])
+                if not op.outputs[0].is_variable:
+                    self.input_tensors.append(op.outputs[0])
 
             for out in op.outputs:
                 if out not in visit_tensor_set:
@@ -535,7 +541,6 @@ class Graph:
         self.metadata = []
         self.memory_used = {}
         self.total_original_weights = 0
-        self.total_npu_weights = 0
         self.total_npu_encoded_weights = 0
         self.weight_cache = None  # See CompressedWeightCache
         self.bandwidths = 0
