@@ -1,4 +1,4 @@
-# Copyright (C) 2020-2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2022 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # NPU performance estimation functions to estimate performance of a Pass and CascadedPass. Uses a model that takes the
 # maximum of the 'cycles required for bandwidth' and 'cycles required for computing'.
@@ -20,6 +21,7 @@
 # Called during scheduling to evaluate different proposals, as well as post-scheduling to provide a final performance
 # estimate.
 import copy
+import csv
 from enum import auto
 from enum import IntEnum
 from typing import Optional
@@ -703,14 +705,15 @@ def estimate_full_op_performance(
     )
 
     # IFM read
-    ifm = op.parent_op.ifm
+    ifm = op.parent_op.ifm2 if op.reversed_operands else op.parent_op.ifm
     bw = access.ifm_read[0] * ifm.element_size()
     bws[ifm.mem_area][ifm.purpose][BandwidthDirection.Read] += bw
     scaled_bws[ifm.mem_area][ifm.purpose][BandwidthDirection.Read] += _estimate_memory_transfer_efficiency(
         arch, True, query.ifm_memory_area, ifm.format, query.ifm_bits, query.config.ifm_block, query.ifm_shape, bw
     )
+
     if query.ifm2_shape:
-        ifm2 = op.parent_op.ifm2
+        ifm2 = op.parent_op.ifm if op.reversed_operands else op.parent_op.ifm2
         bw = access.ifm_read[1] * ifm2.element_size()
         bws[ifm2.mem_area][ifm2.purpose][BandwidthDirection.Read] += bw
         scaled_bws[ifm2.mem_area][ifm2.purpose][BandwidthDirection.Read] += _estimate_memory_transfer_efficiency(
@@ -759,7 +762,15 @@ def print_performance(
     macs: dict,
     cycles: dict,
     mem_usage: dict,
+    output_basename: str,
 ):
+    def _percentage(part, whole):
+        # desired behaviour is for division by zero to return 100%
+        if whole == 0:
+            return 100.0
+        else:
+            return part / whole * 100.0
+
     if network_type == NetworkType.TFLite:
         nng_optype_to_input_op_type = tflite_optype_to_builtintype
     else:
@@ -767,67 +778,114 @@ def print_performance(
 
     suid_inv_map = {v: k for k, v in DebugDatabase._sourceUID.items()}
 
+    # the header is a list (one entry per column) of tuples (column name, alignment, width, precision)
+    header = [
+        (f"{network_type.name}_operator", "<", 20, -1),
+        ("NNG Operator", "<", 20, -1),
+        ("SRAM Usage", ">", 10, 0.0),
+        ("Peak%", ">", 6, 0.2),
+        ("Op Cycles", ">", 10, 0.0),
+        ("Network%", ">", 8, 0.2),
+        ("NPU", ">", 10, 0.0),
+        ("SRAM AC", ">", 10, 0.0),
+        ("DRAM AC", ">", 10, 0.0),
+        ("OnFlash AC", ">", 10, 0.0),
+        ("OffFlash AC", ">", 11, 0.0),
+        ("MAC Count", ">", 10, 0.0),
+        ("Network%", ">", 8, 0.2),
+        ("Util%", ">", 6, 0.2),
+        ("Name", "<", 20, -1),
+    ]
+
+    # open the csv
+    csv_file = open(output_basename + "_per-layer.csv", "w", encoding="UTF8")
+    writer = csv.writer(csv_file)
+
     for sg in nng.subgraphs:
 
         if sg.placement != PassPlacement.Npu:
             continue
 
-        print(f"\n{str('#') * 80}")
-        print(f"Performance for NPU Subgraph {sg.name}")
-        print(
-            f" {network_type.name + str(' Operator:'):20s}"
-            f" {str('NNG Operator:'):20s}"
-            f" {str('SRAM Usage'):>10s}"
-            f" ({str('Peak'):>6s}%):"
-            f"{str('Op Cycles'):>10s}"
-            f" ({str('Netwrk'):>6s}%)"
-            f" ["
-            f" {str('NPU'):>10s}"
-            f" {str('SRAM AC'):>10s}"
-            f" {str('DRAM AC'):>10s}"
-            f" {str('OnFlash AC'):>10s}"
-            f" {str('OffFlashAC'):>10s}"
-            f" ]:"
-            f"{str('MAC Count'):>10s}"
-            f" ({str('Netwrk'):>6s}% / {str('Util'):>6s}%):"
-            f"Name:"
-        )
+        sg_seperator_text = f"\n{str('#') * 80}\nPerformance for NPU Subgraph {sg.name}"
 
+        # the data is a list (one entry per op) of lists (matching the header columns)
+        data = []
         for sched_op in sg.sched_ops:
             # get source op name
             sched_op_src_uid = DebugDatabase._optimisedUID[sched_op.parent_op][1]
             if sched_op_src_uid == DebugDatabase.NULLREF:
                 src_op_type = None
             else:
-                src_op_type = suid_inv_map[sched_op_src_uid].type
+                src_op_type = suid_inv_map[sched_op_src_uid].original_type
 
             src_op_name = nng_optype_to_input_op_type(src_op_type)
 
             max_macs = cycles[sched_op][PassCycles.Total] * arch.num_macs_per_cycle * arch.ncores
             peak_sram = (
-                mem_usage[sched_op] / nng.memory_used[MemArea.Sram] * 100 if MemArea.Sram in nng.memory_used else 0
-            )
-            print(
-                f" {src_op_name:20s}"
-                f" {sched_op.op_type:20s}"
-                f" {mem_usage[sched_op]:10.0f}"
-                f" ({peak_sram:6.2f}%)"
-                f" {cycles[sched_op][PassCycles.Total]:10.0f}"
-                f" ({cycles[sched_op][PassCycles.Total] / nng.cycles[PassCycles.Total] * 100:6.2f}%)"
-                f" ["
-                f" {cycles[sched_op][PassCycles.Npu]:10.0f}"
-                f" {cycles[sched_op][PassCycles.SramAccess]:10.0f}"
-                f" {cycles[sched_op][PassCycles.DramAccess]:10.0f}"
-                f" {cycles[sched_op][PassCycles.OnChipFlashAccess]:10.0f}"
-                f" {cycles[sched_op][PassCycles.OffChipFlashAccess]:10.0f}"
-                f" ]"
-                f" {macs[sched_op]:10d}"
-                f" ({macs[sched_op] / nng.macs * 100:6.2f}% / {macs[sched_op] / max_macs * 100:6.2f}%)"
-                f" {sched_op.name:s}"
+                _percentage(mem_usage[sched_op], nng.memory_used[MemArea.Sram])
+                if MemArea.Sram in nng.memory_used
+                else 0
             )
 
+            data.append(
+                [
+                    src_op_name,
+                    sched_op.op_type,
+                    mem_usage[sched_op],
+                    peak_sram,
+                    cycles[sched_op][PassCycles.Total],
+                    _percentage(cycles[sched_op][PassCycles.Total], nng.cycles[PassCycles.Total]),
+                    cycles[sched_op][PassCycles.Npu],
+                    cycles[sched_op][PassCycles.SramAccess],
+                    cycles[sched_op][PassCycles.DramAccess],
+                    cycles[sched_op][PassCycles.OnChipFlashAccess],
+                    cycles[sched_op][PassCycles.OffChipFlashAccess],
+                    macs[sched_op],
+                    _percentage(macs[sched_op], nng.macs),
+                    _percentage(macs[sched_op], max_macs),
+                    sched_op.name,
+                ]
+            )
 
-def calc_new_performance_for_network(nng: Graph, arch, network_type: NetworkType, verbose_performance: bool):
+        # print to console
+        print(sg_seperator_text)
+        line = ""
+        line2 = ""
+        for col_name, align, width, _ in header:
+            line_data = f"{col_name:{align}{width}}"
+            line += line_data + " "
+            line2 += "-" * len(line_data) + " "
+        print(line)
+        print(line2)
+
+        for op_data in data:
+            line = ""
+            for idx, item in enumerate(op_data):
+                _, align, width, precision = header[idx]
+                if precision == -1:
+                    w = str(width)
+                else:
+                    w = str(width + precision) + "f"
+                line += f"{item:{align}{w}}" + " "
+            print(line)
+
+        # print to csv
+        writer.writerow((sg_seperator_text,))
+        writer.writerow(col_name for col_name, _, _, _ in header)
+        for op_data in data:
+            writer.writerow(op_data)
+
+    # close the csv
+    csv_file.close()
+
+
+def calc_new_performance_for_network(
+    nng: Graph,
+    arch,
+    network_type: NetworkType,
+    verbose_performance: bool,
+    output_basename: str = "output/unnamed_network",
+):
     total_bws = make_bandwidth_array()
     total_macs = 0
     total_cycles = np.zeros(PassCycles.Size)
@@ -886,4 +944,4 @@ def calc_new_performance_for_network(nng: Graph, arch, network_type: NetworkType
     nng.total_npu_encoded_weights = total_encoded_weight_size
 
     if verbose_performance:
-        print_performance(nng, arch, network_type, bws, macs, cycles, mem_usage)
+        print_performance(nng, arch, network_type, bws, macs, cycles, mem_usage, output_basename)
