@@ -1,4 +1,4 @@
-# Copyright (C) 2020-2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # Internal representation of a Neural Network Tensor.
 import copy
@@ -113,7 +114,8 @@ class TensorPurpose(enum.IntFlag):
     ScratchFast = 4
     LUT = 5
     FSBias = 6
-    Size = 7
+    Virtual = 7
+    Size = 8
 
     def display_name(self) -> str:
         return ("Unknown", "Weights", "FeatureMap", "Scratch", "ScratchFast", "LUT", "FastStorageBias", "Size")[
@@ -213,6 +215,7 @@ class QuantizationParameters:
         "max",
         "num_bits",
         "narrow_range",
+        "next_after",
         "scale_f32",
         "zero_point",
         "quant_min",
@@ -233,6 +236,10 @@ class QuantizationParameters:
         self.num_bits = num_bits
         self.narrow_range = narrow_range
 
+        # Use the 'next after' float value of scale_f32 when converting to scale and shift. It can be combined with
+        # natural rounding to perform rounding away from zero. This only affects the ofm scale and bias tensor, it has
+        # no affect on global scaling i.e. the ofm_scale register
+        self.next_after = False
         self.scale_f32: Union[float, np.ndarray, None] = None
         self.zero_point: Union[int, np.ndarray, None] = None
         self.quant_min: Optional[float] = None
@@ -240,12 +247,9 @@ class QuantizationParameters:
         self.quant_dim: Optional[int] = None
 
     def __str__(self):
-        return "<nng.QuantizationParameters min=%s max=%s, num_bits=%s, scale=%s, zero_point=%s>" % (
-            self.min,
-            self.max,
-            self.num_bits,
-            self.scale_f32,
-            self.zero_point,
+        return (
+            f"<nng.QuantizationParameters min={self.min}, max={self.max}, num_bits={self.num_bits}, "
+            f"scale={self.scale_f32}, zero_point={self.zero_point}, next={self.next_after}>"
         )
 
     __repr__ = __str__
@@ -258,6 +262,7 @@ class QuantizationParameters:
         res.num_bits = self.num_bits
         res.narrow_range = self.narrow_range
 
+        res.next_after = self.next_after
         res.scale_f32 = self.scale_f32
         res.zero_point = self.zero_point
         res.quant_min = self.quant_min
@@ -293,20 +298,42 @@ class QuantizationParameters:
         return False
 
 
+def create_virtual_tensor(
+    name: str,
+):
+    virtual_tensor = Tensor([], DataType.int8, name)
+    virtual_tensor.purpose = TensorPurpose.Virtual
+    return virtual_tensor
+
+
 def create_const_tensor(
     name: str,
     shape: Shape,
-    dtype: DataType,
-    values: np.ndarray,
-    value_dtype: np.dtype = None,
+    dtype: DataType,  # datatype of the tensor
+    values: Optional[Union[np.ndarray, list]],  # list-like data of some type, or scalar (skip mypy), or None
     purpose: TensorPurpose = TensorPurpose.Unknown,
-    quantization: QuantizationParameters = None,
+    quantization: Optional[QuantizationParameters] = None,
 ):
+    assert isinstance(dtype, DataType)
+
     # Tensor
     const_tensor = Tensor(shape, dtype, name + "_0")
     const_tensor.purpose = purpose
     const_tensor.quantization = quantization
-    const_tensor.values = np.array(values, dtype=value_dtype)
+
+    # if the tensor datatype does not match that of the values then np.array() will perform a cast operation. this can
+    # result in undefined behaviour if casting from a numpy float to a numpy unsigned integer. therefore, we need to
+    # avoid this undefined behaviour by converting the numpy floats to python floats as these give the desired behaviour
+    # when casting to unsigned integers
+    if (
+        values is not None
+        and shape != []  # values are not a scalar
+        and isinstance(values[0], np.floating)
+        and dtype.type == BaseType.Unsigned
+    ):
+        values = [float(v) for v in values]
+
+    const_tensor.values = np.array(values, dtype=dtype.as_numpy_type())
     # Operator
     const_op = Operation(Op.Const, name)
     const_op.set_output_tensor(const_tensor)
@@ -337,6 +364,7 @@ class TensorAddressMap:
 class Tensor:
     __slots__ = (
         "shape",
+        "_original_shape",
         "storage_shape",
         "bandwidth_shape",
         "dtype",
@@ -376,6 +404,7 @@ class Tensor:
 
     def __init__(self, shape: Shape, dtype: DataType, name: str):
         self.shape = shape
+        self._original_shape = shape
         self.storage_shape = shape
         self.bandwidth_shape = shape
         self.dtype = dtype
@@ -421,6 +450,10 @@ class Tensor:
         # Reference to parent-tensor if this tensor is a clone
         self.src_tensor: Optional[Tensor] = None
 
+    @property
+    def original_shape(self):
+        return self._original_shape
+
     @property
     def address(self) -> int:
         return TensorAddressMap.get_address_for_tens(self.equivalence_id, self.mem_type)
@@ -433,6 +466,17 @@ class Tensor:
     def is_standard_fm(self) -> bool:
         return self.sub_purpose == TensorSubPurpose.Standard and self.purpose == TensorPurpose.FeatureMap
 
+    @property
+    def is_const(self) -> bool:
+        return self.ops != [] and self.ops[0].type == Op.Const
+
+    @property
+    def is_scalar(self) -> bool:
+        return self.shape == [] and self.elements() == 1
+
+    def is_broadcast(self, ofm) -> bool:
+        return self.shape != ofm.shape
+
     def element_size(self) -> int:
         if self.element_size_bytes == 0:
             return self.dtype.size_in_bits() // 8
@@ -464,6 +508,17 @@ class Tensor:
         res.src_tensor = self
         return res
 
+    def as_1D(self):
+        self.shape = [np.prod(self.shape)]
+        if self.values is not None:
+            self.values = self.values.reshape(self.shape)
+
+    def transpose(self, reorder):
+        self.shape = [self.shape[idx] for idx in reorder]
+        self._original_shape = [self._original_shape[idx] for idx in reorder]
+        if self.values is not None:
+            self.values = self.values.transpose(reorder)
+
     def copy_compressed_weight_info(self, src_tens: "Tensor"):
         # Copies compressed values + all related weight compression info from the given tensor
         self.equivalence_id = src_tens.equivalence_id
@@ -592,7 +647,9 @@ class Tensor:
         rounding_quantum = full_shape(4, list(self.storage_rounding_quantum), 1)
         return Shape4D(shape_round_to_quantum(op_shape4D.as_list(), rounding_quantum))
 
-    def addresses_for_rolling_buffer(self, start_coord: Shape, end_coord: Shape, op_shape4D: Shape4D) -> Tuple:
+    def addresses_for_rolling_buffer(
+        self, start_coord: Shape, end_coord: Shape, strides: List[int], op_shape4D: Shape4D
+    ) -> Tuple:
         # returns ( box_height0, box_height1, box_width, [address_tl, address_tr, address_bl, address_br] )
 
         if self.storage_shape == []:
@@ -600,7 +657,7 @@ class Tensor:
                 1,
                 1,
                 1,
-                [self.address_for_coordinate(start_coord, op_shape4D=op_shape4D), None, None, None],
+                [self.address_for_coordinate(start_coord, strides, op_shape4D), 0, 0, 0],
             )
 
         if self.is_standard_fm:
@@ -617,68 +674,29 @@ class Tensor:
         box_height0 = crossing_y - start_coord[1]
         box_width = crossing_x - start_coord[2]
 
-        addresses: List = [None] * 4
-        addresses[0] = self.address_for_coordinate(start_coord, op_shape4D=op_shape4D)
+        addresses: List = [0] * 4
+        addresses[0] = self.address_for_coordinate(start_coord, strides, op_shape4D)
 
         if end_coord[2] > crossing_x:
             addresses[1] = self.address_for_coordinate(
-                [start_coord[0], start_coord[1], crossing_x, start_coord[3]], op_shape4D=op_shape4D
+                [start_coord[0], start_coord[1], crossing_x, start_coord[3]], strides, op_shape4D
             )
             raise UnsupportedFeatureError("Striping in vertical direction is not supported")
         if end_coord[1] > crossing_y:
             addresses[2] = self.address_for_coordinate(
-                [start_coord[0], crossing_y, start_coord[2], start_coord[3]], op_shape4D=op_shape4D
+                [start_coord[0], crossing_y, start_coord[2], start_coord[3]], strides, op_shape4D
             )
         if end_coord[1] > crossing_y and end_coord[2] > crossing_x:
             addresses[3] = self.address_for_coordinate(
-                [start_coord[0], crossing_y, crossing_x, start_coord[3]], op_shape4D=op_shape4D
+                [start_coord[0], crossing_y, crossing_x, start_coord[3]], strides, op_shape4D
             )
 
         return box_height0, box_height0, box_width, addresses
 
-    def address_for_coordinate(self, coord: Shape, is_top_box: bool = False, op_shape4D: Shape4D = None) -> int:
-        offset = self.address_offset_for_coordinate(coord, op_shape4D=op_shape4D, is_top_box=is_top_box)
-        assert offset is not None
-        return self.address + offset
-
-    def get_strides_and_coord(
-        self, coord: Optional[Shape] = None, shape4D: Optional[Shape4D] = None
-    ) -> Tuple[Optional[Shape], Optional[Shape]]:
-        if coord is None:
-            coord = [0] * min(len(self.storage_shape), 4)
-
-        if shape4D and self.is_standard_fm:
-            augmented_shape = self.get_4D_storage_shape_for_shape(shape4D).as_list()
-        else:
-            augmented_shape = full_shape(4, self.storage_shape, 1)
-
-        augmented_coord = coord
-
-        while len(augmented_coord) < 4:
-            augmented_coord = [0] + augmented_coord
-
-        assert len(augmented_coord) == len(augmented_shape)
-
-        if self.format == TensorFormat.NHWC:
-            augmented_shape = [augmented_shape[0], augmented_shape[3]] + augmented_shape[1:3] + [1]
-            augmented_coord = [augmented_coord[0], augmented_coord[3]] + augmented_coord[1:3] + [0]
-
-        elif self.format == TensorFormat.NHCWB16:
-            channel_divisor = 16
-            augmented_shape = augmented_shape[0:4] + [1]
-            augmented_coord = (
-                [augmented_coord[0], augmented_coord[3] // channel_divisor]
-                + augmented_coord[1:3]
-                + [augmented_coord[3] % channel_divisor]
-            )
-
-            if augmented_shape[1] == 0:
-                augmented_shape[1] = 1
-
-        else:
-            assert self.format in (TensorFormat.Unknown, TensorFormat.WeightsCompressed)
-            return None, None
+    def get_strides(self, shape4D: Optional[Shape4D]) -> List[int]:
 
+        augmented_shape = self.get_augmented_shape(shape4D)
+        assert len(augmented_shape) == 5
         strides: List = [0] * len(augmented_shape)
         stride = self.element_size() * self.storage_compression_scale
 
@@ -688,20 +706,59 @@ class Tensor:
                 strides[i] = stride
                 stride *= augmented_shape[i]
         else:
-            assert len(strides) == 5
             strides[4] = stride
             strides[3] = 16 * stride  # STRIDE_X
             strides[1] = strides[3] * augmented_shape[2]  # STRIDE_C
             strides[2] = augmented_shape[2] * augmented_shape[3] * stride  # STRIDE_Y
             strides[0] = strides[2] * augmented_shape[1]  # STRIDE_N
 
-        return strides, augmented_coord
-
-    def get_strides(self, shape4D: Optional[Shape4D] = None) -> Shape:
-        strides, _ = self.get_strides_and_coord(shape4D=shape4D)
-        assert strides is not None
         return strides
 
+    def get_augmented_shape(self, shape4D: Optional[Shape4D] = None) -> Optional[Shape]:
+
+        if shape4D and self.is_standard_fm:
+            augmented_shape = self.get_4D_storage_shape_for_shape(shape4D).as_list()
+        else:
+            augmented_shape = full_shape(4, self.storage_shape, 1)
+
+        if self.format == TensorFormat.NHWC:
+            augmented_shape = [augmented_shape[0], augmented_shape[3]] + augmented_shape[1:3] + [1]
+
+        elif self.format == TensorFormat.NHCWB16:
+            augmented_shape = augmented_shape[0:4] + [1]
+
+            if augmented_shape[1] == 0:
+                augmented_shape[1] = 1
+
+        else:
+            assert self.format in (TensorFormat.Unknown, TensorFormat.WeightsCompressed)
+            return None
+
+        return augmented_shape
+
+    def get_augmented_coord(self, coord: Optional[Shape] = None) -> Optional[Shape]:
+        if coord is None:
+            coord = [0] * min(len(self.storage_shape), 4)
+
+        missing_len = 4 - len(coord)
+        augmented_coord = ([0] * missing_len) + coord
+
+        if self.format == TensorFormat.NHWC:
+            augmented_coord = [augmented_coord[0], augmented_coord[3]] + augmented_coord[1:3] + [0]
+
+        elif self.format == TensorFormat.NHCWB16:
+            channel_divisor = 16
+            augmented_coord = (
+                [augmented_coord[0], augmented_coord[3] // channel_divisor]
+                + augmented_coord[1:3]
+                + [augmented_coord[3] % channel_divisor]
+            )
+        else:
+            assert self.format in (TensorFormat.Unknown, TensorFormat.WeightsCompressed)
+            return None
+
+        return augmented_coord
+
     def find_npu_op(self) -> Optional[Operation]:
         # Returns the NPU operator that uses this tensor
         for op in self.consumers():
@@ -743,20 +800,32 @@ class Tensor:
         assert 0 <= index < len(self.compressed_values)
         return index == len(self.compressed_values) - 1
 
-    def address_offset_for_coordinate(
-        self, orig_coord: Shape, op_shape4D: Optional[Shape4D] = None, is_top_box: bool = False
+    def address_for_coordinate(
+        self,
+        orig_coord: Shape,
+        strides: Optional[List[int]] = None,
+        op_shape4D: Optional[Shape4D] = None,
+        is_top_box: bool = False,
     ) -> Optional[int]:
+
         address_offset = 0
         assert self.purpose != TensorPurpose.Weights
 
+        # Strides may be passed as an argument, for example when creating feature maps as the strides may be modified
+        # by the "ofm_stride_multiplier" operation attribute. If not, they are calculated here.
+        if not strides:
+            strides = self.get_strides(op_shape4D)
+
+        coord = orig_coord
+        if is_top_box:
+            coord = [c - 1 for c in orig_coord]
+            address_offset += 1 * strides[-1]  # one element
+
         if self.sub_purpose == TensorSubPurpose.Standard:
             shape = op_shape4D.as_list() if op_shape4D else self.shape
-            for idx, c in enumerate(orig_coord):
-                if is_top_box:
-                    assert c > 0 and c <= shape[idx]
-                else:
-                    assert c >= 0 and c < shape[idx]
-        coord = orig_coord
+            for _coord, _shape in zip(coord, shape):
+                assert _coord >= 0 and _coord < _shape
+
         if op_shape4D and self.is_standard_fm:
             storage_shape = self.get_4D_storage_shape_for_shape(op_shape4D).as_list()
             storage_size = self.storage_size_for_shape(storage_shape)
@@ -765,24 +834,16 @@ class Tensor:
             coord = coord[-len(storage_shape) :]
             storage_size = self.storage_size()
 
-        if is_top_box:
-            coord = [c - 1 for c in coord]
+        # Handle wraparound for partial buffers. Make sure to do this after subtracting top box
+        coord = [_coord % _shape for _coord, _shape in zip(coord, storage_shape)]
 
-        # handle wraparound for partial buffers. make sure to do this after subtracting top box:
-        coord = [c % storage_shape[idx] for idx, c in enumerate(coord)]
-
-        strides, augmented_coord = self.get_strides_and_coord(coord, op_shape4D)
-        if strides is None:
-            return None
-
-        if is_top_box:
-            address_offset += 1 * strides[-1]  # one element
+        augmented_coord = self.get_augmented_coord(coord)
+        assert augmented_coord is not None
 
         address_offset += np.dot(augmented_coord, strides)
 
-        assert address_offset >= 0
-        assert address_offset <= storage_size
-        return address_offset
+        assert address_offset >= 0 and address_offset <= storage_size
+        return self.address + address_offset
 
     def is_allocated_in_tensor_arena(self, scratch_tensor_mem_area: MemArea) -> bool:
         return (self.mem_area == scratch_tensor_mem_area) and (self.mem_type in (MemType.Scratch, MemType.Scratch_fast))
