@@ -1,4 +1,4 @@
-# Copyright (C) 2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2021-2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -138,7 +138,13 @@ def test_constraint_quant_scale_inf():
     op = testutil.create_op_with_quant_tensors(Op.Relu, [1, 8, 8, 8], [1, 8, 8, 8])
     op.ifm.quantization.scale_f32 = np.float32(1e9)
     op.ofm.quantization.scale_f32 = np.float32(1e-35)
-    assert not semantic_checker.is_operator_semantic_valid(op)
+    # Temporarily ignore overflow in NumPy
+    old_settings = np.seterr(over="ignore", under="ignore")
+    valid = semantic_checker.is_operator_semantic_valid(op)
+    # Reset NumPy settings
+    np.seterr(**old_settings)
+
+    assert not valid
 
 
 def test_constraint_ofm_scale_too_small():
@@ -191,15 +197,49 @@ def test_constraint_beta_value_range():
     assert semantic_checker.is_operator_semantic_valid(op)
 
 
+def test_constraint_split_axis():
+    # Axis value must be in the range [-<ifm_dimensions>, <ifm_dimensions>)"
+    attrs = {"num_splits": 2}
+    axis = create_const_tensor("axis", [1], DataType.int8, [3])
+    ifm = Tensor([1, 1, 4], DataType.int8, "ifm")
+    ifm.quantization = testutil.default_quant_params()
+    ofm = Tensor([1, 1, 4], DataType.int8, "ofm")
+    ofm.quantization = testutil.default_quant_params()
+    op = testutil.create_op(Op.Split, [axis, ifm], ofm, attrs)
+    # Check invalid axis value
+    assert not semantic_checker.is_operator_semantic_valid(op)
+    # Check valid axis value
+    axis = create_const_tensor("axis", [1], DataType.int8, [-1])
+    op = testutil.create_op(Op.Split, [axis, ifm], ofm, attrs)
+    assert semantic_checker.is_operator_semantic_valid(op)
+
+
+def test_constraint_split_num_splits():
+    # Check that split number is valid"
+    attrs = {"num_splits": 2}
+    axis = create_const_tensor("axis", [1], DataType.int8, [-1])
+    ifm = Tensor([1, 1, 3], DataType.int8, "ifm")
+    ifm.quantization = testutil.default_quant_params()
+    ofm = Tensor([1, 1, 3], DataType.int8, "ofm")
+    ofm.quantization = testutil.default_quant_params()
+    op = testutil.create_op(Op.Split, [axis, ifm], ofm, attrs)
+    # Check invalid split number 2
+    assert not semantic_checker.is_operator_semantic_valid(op)
+    # Check valid split number 3
+    attrs = {"num_splits": 3}
+    op = testutil.create_op(Op.Split, [axis, ifm], ofm, attrs)
+    assert semantic_checker.is_operator_semantic_valid(op)
+
+
 def test_constraint_splitv_inferred():
     # SplitV requires a maximum of one inferred shape (-1)
     qp = testutil.default_quant_params()
     op = testutil.create_op_with_quant_tensors(Op.SplitV, [1, 1, 1, 8], [1, 1, 1, 8])
-    sizes = create_const_tensor("sizes", [1, 1, 1, 4], DataType.int16, [[[[0, -1, 2, -1]]]], np.int16, quantization=qp)
+    sizes = create_const_tensor("sizes", [1, 1, 1, 4], DataType.int16, [[[[0, -1, 2, -1]]]], quantization=qp)
     op.add_input_tensor(sizes)
     assert not semantic_checker.is_operator_semantic_valid(op)
     op = testutil.create_op_with_quant_tensors(Op.SplitV, [1, 1, 1, 8], [1, 1, 1, 8])
-    sizes = create_const_tensor("sizes", [1, 1, 1, 4], DataType.int16, [[[[0, 1, 2, -1]]]], np.int16, quantization=qp)
+    sizes = create_const_tensor("sizes", [1, 1, 1, 4], DataType.int16, [[[[0, 1, 2, -1]]]], quantization=qp)
     op.add_input_tensor(sizes)
     assert semantic_checker.is_operator_semantic_valid(op)
 
@@ -278,7 +318,8 @@ def create_pad_op(
     qp = testutil.default_quant_params()
     in0 = Tensor(in_shape, in_dtype, "in")
     in0.quantization = qp
-    pad_tensor = create_const_tensor(name="pad", shape=list(np.shape(padding)), values=padding, dtype=pad_dtype)
+    shape = [] if padding == [] else list(np.shape(padding))
+    pad_tensor = create_const_tensor(name="pad", shape=shape, values=padding, dtype=pad_dtype)
     out = Tensor(out_shape, out_dtype, "out")
     out.quantization = qp.clone()
     op = testutil.create_op(Op.Pad, [in0, pad_tensor], out)
@@ -412,20 +453,6 @@ def test_constraint_matching_either_shapes():
     assert not semantic_checker.is_operator_semantic_valid(op)
 
 
-def test_constraint_alpha_valid():
-    # Alpha can only be negative for int8 and uint8
-    op = testutil.create_elemwise_op(Op.LeakyRelu, "op", [2, 2], None, [2, 2], DataType.int16)
-    op.attrs["alpha"] = 0
-    assert semantic_checker.is_operator_semantic_valid(op)
-    op.attrs["alpha"] = -1
-    assert not semantic_checker.is_operator_semantic_valid(op)
-    op = testutil.create_elemwise_op(Op.LeakyRelu, "op", [2, 2], None, [2, 2], DataType.int8)
-    op.attrs["alpha"] = 0
-    assert semantic_checker.is_operator_semantic_valid(op)
-    op.attrs["alpha"] = -1
-    assert semantic_checker.is_operator_semantic_valid(op)
-
-
 def test_constraint_hardswish_dtype():
     # HardSwish operator dtype should be int8 or uint8, and input dtype must match output
     # UINT8
@@ -463,9 +490,9 @@ def create_mean(input_shape, output_shape, axis, datatype, attrs):
     ofm = Tensor(output_shape, datatype, "out")
     ofm.quantization = testutil.default_quant_params()
     if type(axis) is list:
-        indices = create_const_tensor("indices", [len(axis)], DataType.int32, axis, np.uint8)
+        indices = create_const_tensor("indices", [len(axis)], DataType.int32, axis)
     elif type(axis) is int:
-        indices = create_const_tensor("indices", [], DataType.int32, axis, np.uint8)
+        indices = create_const_tensor("indices", [], DataType.int32, axis)
     op = testutil.create_op(Op.Mean, [ifm, indices], ofm, attrs)
     return op
 
