@@ -1,4 +1,4 @@
-# Copyright (C) 2020-2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -17,6 +17,7 @@
 # Description:
 # Unit tests for tflite support_operators
 import numpy as np
+import pytest
 
 from ethosu.vela.data_type import DataType
 from ethosu.vela.operation import ActivationFunction
@@ -62,7 +63,7 @@ def test_constraint_tens_quant_per_axis_not_supp():
 
 def test_constraint_tens_quant_per_axis_is_supp():
     op = testutil.create_op_with_quant_tensors(
-        Op.Conv2DBias, [1, 1, 1, 3], [1, 1, 1, 3], weights_shape=[1, 1, 1, 3], bias_shape=[1, 1, 1, 3]
+        Op.Conv2DBias, [1, 1, 1, 3], [1, 1, 1, 3], weights_shape=[1, 1, 1, 3], bias_shape=[3]
     )
     op.attrs = {"stride_w": 1, "stride_h": 1}
     assert support.is_operator_supported(op)
@@ -104,18 +105,26 @@ def test_constraint_conv_pass():
     assert support.is_operator_supported(op)
 
 
-def test_constraint_stride_range():
+@pytest.mark.parametrize(
+    "stride_w, stride_h, supported",
+    [
+        [0, 20, False],
+        [4, 1, True],
+        [4, 2, True],
+        [2, 2, True],
+        [4, 4, False],
+        [4, 5, False],
+        [5, 4, False],
+        [3, 3, True],
+        [1, 1, True],
+        [2, 4, False],
+    ],
+)
+def test_constraint_stride_range(stride_w: int, stride_h: int, supported: bool):
     # Stride width and height must lie within a certain range
-    op = testutil.create_op_with_quant_tensors(Op.Conv2DBias, [1, 8, 8, 8], [1, 8, 8, 8])
-    op.attrs = {"stride_w": 0, "stride_h": 20}
-    assert not support.is_operator_supported(op)
-
-
-def test_constraint_dilation_range():
-    # Dilation width and height must lie within a certain range
-    op = testutil.create_op_with_quant_tensors(Op.Conv2DBias, [1, 8, 8, 8], [1, 8, 8, 8])
-    op.attrs = {"stride_w": 1, "stride_h": 1, "dilation_w_factor": 0, "dilation_h_factor": 20}
-    assert not support.is_operator_supported(op)
+    op = testutil.create_op_with_quant_tensors(Op.Conv2DBias, [1, 8, 8, 8], [1, 8, 8, 8], [1, 1, 1, 1])
+    op.attrs = {"stride_w": stride_w, "stride_h": stride_h}
+    assert support.is_operator_supported(op) == supported
 
 
 def test_constraint_dilated_height_range():
@@ -310,55 +319,55 @@ def test_constraint_resize():
     for resize_op in Op.op_set(Op.is_resize_op):
         # IFM W and H == 1
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 1, 1, 8], [1, 8, 8, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8]))
         assert support.is_operator_supported(op)
 
         # IFM == OFM
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 8, 8, 8], [1, 8, 8, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8]))
         assert support.is_operator_supported(op)
 
         # IFM x2 == OFM ; align_corners = False
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 8, 8, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8]))
         assert support.is_operator_supported(op)
 
         # IFM x4 == OFM ; align_corners = False
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 16, 16, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [16, 16], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [16, 16]))
         assert support.is_operator_supported(op)
 
         # IFM x8 == OFM ; align_corners = False
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 32, 32, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [32, 32], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [32, 32]))
         assert support.is_operator_supported(op)
 
         # IFM -1 x2 == OFM -1 ; align_corners = True
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 7, 7, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [7, 7], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [7, 7]))
         op.attrs["align_corners"] = True
         assert support.is_operator_supported(op)
 
         # IFM -1 x4 == OFM -1 ; align_corners = True
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 13, 13, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [13, 13], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [13, 13]))
         op.attrs["align_corners"] = True
         assert support.is_operator_supported(op)
 
         # IFM -1 x8 == OFM -1 ; align_corners = True
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 25, 25, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [25, 25], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [25, 25]))
         op.attrs["align_corners"] = True
         assert support.is_operator_supported(op)
 
         # Invalid case - upscale size
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 17, 17, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [17, 17], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [17, 17]))
         assert not support.is_operator_supported(op)
 
         # Invalid case - upscale size with align corners
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 15, 15, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [15, 15], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [15, 15]))
         op.attrs["align_corners"] = True
         assert not support.is_operator_supported(op)
 
@@ -367,7 +376,7 @@ def test_constraint_resize_size():
     for resize_op in Op.op_set(Op.is_resize_op):
         # Invalid case - size != ofm size
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 8, 8, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [7, 7], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [7, 7]))
         assert not support.is_operator_supported(op)
 
 
@@ -375,21 +384,12 @@ def test_constraint_resize_attrs():
     for resize_op in Op.op_set(Op.is_resize_op):
         # Invalid case - both align corners and half-pixel centers
         op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 8, 8, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8], np.int32))
+        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8]))
         op.attrs["align_corners"] = True
         op.attrs["half_pixel_centers"] = True
         assert not support.is_operator_supported(op)
 
 
-def test_constraint_resize_half_pixel_centers():
-    for resize_op in Op.op_set(Op.is_resize_op):
-        # Invalid case - half-pixel centers (not supported)
-        op = testutil.create_op_with_quant_tensors(resize_op, [1, 4, 4, 8], [1, 8, 8, 8])
-        op.add_input_tensor(create_const_tensor("size", [2], DataType.int32, [8, 8], np.int32))
-        op.attrs["half_pixel_centers"] = True
-        assert not support.is_operator_supported(op)
-
-
 def test_constraint_concat_pass():
     # A working concat
     op = testutil.create_op_with_quant_tensors(Op.Concat, [1, 1, 1, 4], [1, 1, 1, 8])
@@ -411,7 +411,8 @@ def create_pad_op(
     qp = testutil.default_quant_params()
     in0 = Tensor(in_shape, in_dtype, "in")
     in0.quantization = qp
-    pad_tensor = create_const_tensor(name="pad", shape=list(np.shape(padding)), values=padding, dtype=pad_dtype)
+    shape = [] if padding == [] else list(np.shape(padding))
+    pad_tensor = create_const_tensor(name="pad", shape=shape, values=padding, dtype=pad_dtype)
     out = Tensor(out_shape, out_dtype, "out")
     out.quantization = qp.clone()
     op = testutil.create_op(Op.Pad, [in0, pad_tensor], out)
@@ -547,25 +548,25 @@ def test_constraint_matching_quantization_parameters():
 
 def test_constraint_elemwise_batch_size():
     # BINARY CASE
-    # Batch can be >1 if dims is <=2D
-    op = testutil.create_elemwise_op(Op.Add, "op", [2, 2], [2, 2], [2, 2])
+    # Batch can be >1 if dims is <=3D
+    op = testutil.create_elemwise_op(Op.Add, "op", [2, 2, 2], [2, 2, 2], [2, 2, 2])
     assert support.is_operator_supported(op)
-    # For dims >2D, batch must be 1
-    op = testutil.create_elemwise_op(Op.Add, "op", [1, 2, 2], [1, 2, 2], [1, 2, 2])
+    # For dims >3D, batch must be 1
+    op = testutil.create_elemwise_op(Op.Add, "op", [1, 2, 2, 2], [1, 2, 2, 2], [1, 2, 2, 2])
     assert support.is_operator_supported(op)
     # invalid case
-    op = testutil.create_elemwise_op(Op.Add, "op", [2, 2, 2], [2, 2, 2], [2, 2, 2])
+    op = testutil.create_elemwise_op(Op.Add, "op", [2, 2, 2, 2], [2, 2, 2, 2], [2, 2, 2, 2])
     assert not support.is_operator_supported(op)
 
     # UNARY CASE
-    # Batch can be >1 if dims is <=2D
-    op = testutil.create_elemwise_op(Op.CLZ, "op", [2, 2], None, [2, 2], datatype=DataType.int32)
+    # Batch can be >1 if dims is <=3D
+    op = testutil.create_elemwise_op(Op.CLZ, "op", [2, 2, 2], None, [2, 2, 2], datatype=DataType.int32)
     assert support.is_operator_supported(op)
-    # For dims >2D, batch must be 1
-    op = testutil.create_elemwise_op(Op.CLZ, "op", [1, 2, 2], None, [1, 2, 2], datatype=DataType.int32)
+    # For dims >3D, batch must be 1
+    op = testutil.create_elemwise_op(Op.CLZ, "op", [1, 2, 2, 2], None, [1, 2, 2, 2], datatype=DataType.int32)
     assert support.is_operator_supported(op)
     # invalid case
-    op = testutil.create_elemwise_op(Op.CLZ, "op", [2, 2, 2], None, [2, 2, 2], datatype=DataType.int32)
+    op = testutil.create_elemwise_op(Op.CLZ, "op", [2, 2, 2, 2], None, [2, 2, 2, 2], datatype=DataType.int32)
     assert not support.is_operator_supported(op)
 
 
@@ -603,9 +604,9 @@ def create_mean(input_shape, output_shape, axis, datatype, attrs):
     ofm = Tensor(output_shape, datatype, "out")
     ofm.quantization = testutil.default_quant_params()
     if type(axis) is list:
-        indices = create_const_tensor("indices", [len(axis)], DataType.int32, axis, np.uint8)
+        indices = create_const_tensor("indices", [len(axis)], DataType.int32, axis)
     elif type(axis) is int:
-        indices = create_const_tensor("indices", [], DataType.int32, axis, np.uint8)
+        indices = create_const_tensor("indices", [], DataType.int32, axis)
     op = testutil.create_op(Op.Mean, [ifm, indices], ofm, attrs)
     return op
 
@@ -617,13 +618,6 @@ def test_mean_hw_product():
     assert not support.is_operator_supported(op)
 
 
-def test_mean_hw_product_int8():
-    op = create_mean([1, 16, 16, 16], [1, 1, 1, 16], [1, 2], DataType.int8, {"keep_dims": True})
-    assert support.is_operator_supported(op)
-    op = create_mean([1, 16, 17, 16], [1, 1, 1, 16], [1, 2], DataType.int8, {"keep_dims": True})
-    assert not support.is_operator_supported(op)
-
-
 def test_mean_hw_product_avgpool():
     op = create_mean([1, 200, 200, 16], [1, 16], [1, 2], DataType.uint8, {"keep_dims": False})
     assert support.is_operator_supported(op)
