@@ -1,4 +1,4 @@
-# Copyright (C) 2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2021-2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # The TFLiteSemantic class which is a collection of TensorFlow lite model semantic checks.
 from collections import defaultdict
@@ -22,7 +23,6 @@ import numpy as np
 from .data_type import BaseType
 from .data_type import DataType
 from .numeric_util import is_integer
-from .operation import get_slice_offsets
 from .operation import Op
 from .supported_operators_util import docstring_format_args
 from .supported_operators_util import list_formatter
@@ -106,7 +106,9 @@ class TFLiteSemantic:
         # Conv-like checks:
         for op_type in TFLiteSemantic.convolution_like_ops:
             self.specific_constraints[op_type].append(TFLiteSemantic.constraint_stride_type)
-            self.specific_constraints[op_type].append(TFLiteSemantic.constraint_dilation_type)
+            if op_type not in TFLiteSemantic.transpose_convolution_ops:
+                # Transpose Conv does not contain dilation
+                self.specific_constraints[op_type].append(TFLiteSemantic.constraint_dilation_type)
 
         # Pooling checks:
         for op_type in TFLiteSemantic.pooling_ops:
@@ -126,6 +128,7 @@ class TFLiteSemantic:
             self.specific_constraints[op_type].append(TFLiteSemantic.constraint_axis_valid)
             self.specific_constraints[op_type].append(TFLiteSemantic.constraint_matching_dimensionality)
             self.specific_constraints[op_type].append(TFLiteSemantic.constraint_valid_dimensions)
+            self.specific_constraints[op_type].append(TFLiteSemantic.constraint_valid_dimensions_axis)
 
         # Element-wise checks:
         for op_type in TFLiteSemantic.elem_wise_main_ops:
@@ -151,6 +154,10 @@ class TFLiteSemantic:
         self.specific_constraints[Op.Softmax].append(TFLiteSemantic.constraint_matching_in_out_types)
         self.specific_constraints[Op.Softmax].append(TFLiteSemantic.constraint_beta_value_range)
 
+        # Split specific checks:
+        self.specific_constraints[Op.Split].append(TFLiteSemantic.constraint_split_axis)
+        self.specific_constraints[Op.Split].append(TFLiteSemantic.constraint_split_num_splits)
+
         # SplitV specific checks:
         self.specific_constraints[Op.SplitV].append(TFLiteSemantic.constraint_splitv_inferred)
 
@@ -161,9 +168,6 @@ class TFLiteSemantic:
         self.specific_constraints[Op.StridedSlice].append(TFLiteSemantic.constraint_axis_masks)
         self.specific_constraints[Op.StridedSlice].append(TFLiteSemantic.constraint_slice_ranges)
 
-        # LeakyRelu specific checks:
-        self.specific_constraints[Op.LeakyRelu].append(TFLiteSemantic.constraint_alpha_valid)
-
         # FullyConnected specific checks:
         self.specific_constraints[Op.FullyConnected].append(TFLiteSemantic.constraint_fc_output_2d)
         self.specific_constraints[Op.FullyConnected].append(TFLiteSemantic.constraint_keep_dim_ifm_ofm)
@@ -175,6 +179,7 @@ class TFLiteSemantic:
         # HardSwish specific checks:
         self.specific_constraints[Op.HardSwish].append(TFLiteSemantic.constraint_input_8bit)
         self.specific_constraints[Op.HardSwish].append(TFLiteSemantic.constraint_matching_in_out_types)
+
         # Mean specific checks:
         self.specific_constraints[Op.Mean].append(TFLiteSemantic.constraint_input_8bit)
         self.specific_constraints[Op.Mean].append(TFLiteSemantic.constraint_mean_input_dims)
@@ -393,6 +398,29 @@ class TFLiteSemantic:
         valid = ifm_shape == ofm_shape
         return valid, f"Op has ifm_shape={ifm_shape} and ofm_shape={ofm_shape}"
 
+    @staticmethod
+    def constraint_split_axis(op):
+        "Axis value must be in the range [-RANK(IFM) to +RANK(IFM))"
+        axis_tens = op.inputs[0]
+        input_tens = op.inputs[1]
+        dims = len(input_tens.shape)
+        axis = int(axis_tens.values)
+        axis += dims if axis < 0 else 0
+        valid = 0 <= axis < dims
+        return valid, f"Op has ifm_dimensions={dims} and axis value is: {axis}"
+
+    @staticmethod
+    def constraint_split_num_splits(op):
+        "Axis must be divisible by number of splits"
+        num_splits = op.attrs.get("num_splits")
+        axis_tens = op.inputs[0]
+        input_tens = op.inputs[1]
+        dims = len(input_tens.shape)
+        axis = int(axis_tens.values)
+        axis += dims if axis < 0 else 0
+        valid = input_tens.shape[axis] % num_splits == 0
+        return valid, f"Op has ifm shape={input_tens.shape} axis={axis} num_splits={num_splits}"
+
     @staticmethod
     def constraint_splitv_inferred(op):
         "Only one size is allowed to be inferred"
@@ -448,6 +476,26 @@ class TFLiteSemantic:
         extra = ", ".join(extra)
         return valid, f"Op has axis={axis}, ofm_shape={ofm_shape} and the list of mismatching inputs are: {extra}"
 
+    @staticmethod
+    def constraint_valid_dimensions_axis(op):
+        """The size of the OFM axis must match the sum of all IFM axis defined by the axis attribute"""
+        valid = True
+        extra = []
+        ofm_shape = op.ofm.shape
+        ofm_dim = len(ofm_shape)
+        axis = op.attrs["axis"]
+        axis += ofm_dim if axis < 0 else 0
+
+        sum_ifm_axis = 0
+        tensors = [tens for tens in op.inputs if tens]
+        for tens in tensors:
+            sum_ifm_axis += tens.shape[axis]
+            extra.append(f"Tensor '{tens.name}' has shape: {tens.shape}")
+
+        valid = sum_ifm_axis == ofm_shape[axis]
+        extra = ", ".join(extra)
+        return valid, f"Op has axis={axis}, ofm_shape={ofm_shape} and the list of mismatching inputs are: {extra}"
+
     @staticmethod
     def constraint_stridedslice_input_count(op):
         "Exactly 4 Input tensors are required"
@@ -502,15 +550,42 @@ class TFLiteSemantic:
         valid = (new_axis == 0) or (shrink_axis == 0)
         return valid, f"Op has new_axis_mask={new_axis} and shrink_axis_mask={shrink_axis}"
 
+    def _get_slice_offsets(input_shape, offset_tens, offset_mask, is_begin=True):
+        # For strided slice operator: get start or end offsets
+        # input_shape: List[int], offset_tens: Tensor, offset_mask: int, is_begin: bool = True
+        offsets = len(input_shape) * [0] if is_begin else input_shape[:]
+        for idx in range(len(input_shape)):
+            # If the i:th bit in the mask is not set then the value in offset_tens[i] should be used, otherwise it
+            # should be ignored
+            if (offset_mask & (1 << idx)) == 0:
+                offsets[idx] = offset_tens.values[idx]
+                if offsets[idx] < 0:
+                    # Convert negative indexing to positive ones
+                    offsets[idx] += input_shape[idx]
+        return offsets
+
     @staticmethod
     def constraint_slice_ranges(op):
         "Slice 'end' values must be greater than 'begin' values"
         ifm, begin, end, _ = op.inputs
+        shrink_axis_mask = op.attrs["shrink_axis_mask"]
         # Calculate offset begin/end
-        offset_begin = get_slice_offsets(ifm.shape, begin, op.attrs["begin_mask"], is_begin=True)
-        offset_end = get_slice_offsets(ifm.shape, end, op.attrs["end_mask"], is_begin=False)
+        offset_begin = TFLiteSemantic._get_slice_offsets(ifm.shape, begin, op.attrs["begin_mask"], is_begin=True)
+        offset_end = TFLiteSemantic._get_slice_offsets(ifm.shape, end, op.attrs["end_mask"], is_begin=False)
         # Check "end - begin" doesn't result in any zero or negative elements
-        valid = all((e - b) > 0 for b, e in zip(offset_begin, offset_end))
+        valid = True
+        # if a shrink mask bit is set then the end position provided by the operation should be ignored, and instead a
+        # new end position should be calculated so that calculations in the graph optimiser, such as (end - start),
+        # result in the correct value. otherwise, we just need to check that the begin and end values are valid
+        for i in range(len(ifm.shape)):
+            if (shrink_axis_mask & (1 << i)) != 0:
+                offset_end[i] = offset_begin[i] + 1
+            else:
+                if offset_end[i] <= offset_begin[i]:
+                    valid = False
+
+        op.attrs["offset_begin"] = offset_begin
+        op.attrs["offset_end"] = offset_end
         return valid, f"Op has begin_values={begin.values} and end_values={end.values}"
 
     @staticmethod
@@ -557,14 +632,6 @@ class TFLiteSemantic:
         valid = (ifm_shape == ofm_shape) or (ifm2_shape == ofm_shape)
         return valid, f"Op has ifm_shape={ifm_shape}, ifm2_shape={ifm2_shape} and ofm_shape={ofm_shape}"
 
-    @staticmethod
-    def constraint_alpha_valid(op):
-        "Alpha only allowed to be negative if IFM is int8 or uint8"
-        alpha = op.attrs["alpha"]
-        ifm_dtype = op.ifm.dtype
-        valid = ifm_dtype == DataType.int8 or ifm_dtype == DataType.uint8 or alpha >= 0
-        return valid, f"Op has alpha={alpha} and ifm_dtype={ifm_dtype} "
-
     @staticmethod
     def constraint_keep_dim_ifm_ofm(op):
         "The IFM and OFM must have the same number of dimensions if keep_num_dims is set to true"
