@@ -1,4 +1,4 @@
-# Copyright (C) 2020-2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # Main entry point for the Vela compiler.
 #
@@ -45,6 +46,7 @@ from .tensor import Tensor
 from .tflite.Model import Model
 from .tflite_mapping import builtin_operator_map
 from .tflite_mapping import builtin_operator_name_map
+from .tflite_mapping import optype_to_builtintype
 from .tflite_model_semantic import TFLiteSemantic
 from .tflite_supported_operators import TFLiteSupportedOperators
 from .tosa_model_semantic import TosaSemantic
@@ -75,7 +77,7 @@ def process(input_name, enable_debug_db, arch, model_reader_options, compiler_op
         print("Model reading took %f s" % (stop - start))
         start = time.time()
 
-    compiler_driver.compiler_driver(nng, arch, compiler_options, scheduler_options, network_type)
+    compiler_driver.compiler_driver(nng, arch, compiler_options, scheduler_options, network_type, output_basename)
 
     summary_csv_file = "{0}_summary_{1}.csv".format(output_basename, arch.system_config)
     stats_writer.write_summary_metrics_csv(nng, summary_csv_file, arch)
@@ -178,6 +180,12 @@ def generate_supported_ops():
     # To easily exclude NetworkType from generated documentation.
     exclude_generation_network_type_value = [NetworkType.TOSA.value]
 
+    def _exclude_list_names(constraint, exclude_list):
+        constraints_excluded_names = [
+            optype_to_builtintype(op) for op, exclude_constraint in exclude_list if constraint in exclude_constraint
+        ]
+        return f" - [{', '.join(sorted(constraints_excluded_names))}]" if constraints_excluded_names else ""
+
     lines = [
         "# Supported Ops",
         "",
@@ -256,20 +264,13 @@ def generate_supported_ops():
         for constraint in semantic_checker.generic_constraints:
             # Markdown needs two spaces at the end of a line to render it as a separate line
             reason = constraint.__doc__.replace("\n", "  \n")
-
             exclude_list = TFLiteSemantic.get_generic_constraint_exclude_list().items()
-            constraints_excluded_names = [
-                op.name for op, exclude_constraint in exclude_list if constraint in exclude_constraint
-            ]
-            excluded_constraint_text = ""
-            if constraints_excluded_names:
-                excluded_constraint_text = f"- [{', '.join(constraints_excluded_names)}]"
-
-            lines.append(f"- {reason} {excluded_constraint_text}")
+            lines.append(f"- {reason}{_exclude_list_names(constraint, exclude_list)}")
         for constraint in supported.generic_constraints:
             # Markdown needs two spaces at the end of a line to render it as a separate line
             reason = constraint.__doc__.replace("\n", "  \n")
-            lines.append(f"- {reason}")
+            exclude_list = supported.generic_constraints_exceptions.items()
+            lines.append(f"- {reason}{_exclude_list_names(constraint, exclude_list)}")
         for op, name in op_constraint_links:
             lines += [
                 "",
@@ -375,6 +376,11 @@ def main(args=None):
             "--show-cpu-operations", action="store_true", help="Show the operations that fall back to the CPU"
         )
         parser.add_argument("--timing", action="store_true", help="Time the compiler doing operations")
+        parser.add_argument(
+            "--force-symmetric-int-weights",
+            action="store_true",
+            help="Forces all zero points to 0 for signed integer weights",
+        )
         parser.add_argument(
             "--accelerator-config",
             type=str,
@@ -552,6 +558,7 @@ def main(args=None):
             show_cpu_operations=args.show_cpu_operations,
             tensor_allocator=args.tensor_allocator,
             timing=args.timing,
+            force_symmetric_int_weights=args.force_symmetric_int_weights,
             output_dir=args.output_dir,
             cpu_tensor_alignment=args.cpu_tensor_alignment,
             hillclimb_max_iterations=args.hillclimb_max_iterations,
