@@ -1,4 +1,4 @@
-# Copyright (C) 2020-2021 Arm Limited or its affiliates. All rights reserved.
+# SPDX-FileCopyrightText: Copyright 2020-2023 Arm Limited and/or its affiliates <open-source-office@arm.com>
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -13,6 +13,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+#
 # Description:
 # Compresses and pads the weigths. It also calculates the scales and packs with the biases.
 from collections import namedtuple
@@ -36,7 +37,28 @@ from .scaling import reduced_quantise_scale
 from .tensor import Tensor
 from .tensor import TensorFormat
 from .tensor import TensorPurpose
-from ethosu import mlw_codec
+
+# Handle any errors thrown by NumPy while importing mlw_codec module
+try:
+    from ethosu import mlw_codec
+except RuntimeError as ex:
+    if "mlw_codec error: module compiled against API version" in str(ex):
+        # Extract API versions from error message
+        matches = [s for s in str(ex).split() if "0x" in s]
+        if len(matches) == 2:
+            # Raise new exception with more detailed message
+            raise ImportError(  # pylint: disable=W0707
+                "NumPy C API version mismatch "
+                f"(Build-time version: {matches[0]}, "
+                f"Run-time version: {matches[1]})"
+                "\nThis is a known issue most likely caused by a change in the API "
+                "version in NumPy after installing ethos-u-vela.\nYou can find more "
+                "information about the issue and possible solutions in the "
+                "'Known Issues' section at https://review.mlplatform.org/"
+                "plugins/gitiles/ml/ethos-u/ethos-u-vela/+/refs/heads/main/"
+                "README.md#known-issues"
+            )
+    raise
 
 
 # Contains meta info for a weight compression. If two tensors have identical weight compression config,
@@ -275,11 +297,18 @@ def _prepare_scale_and_bias(arch, tens, rescale_for_faf, explicit_scaling):
         quantised_scales = [(int(m), int(s)) for s, m in zip(explicit_scaling.shift, explicit_scaling.multiplier)]
     else:
         # quantise all of the weight scales into (scale_factor, shift)
-        if ifm_dtype == DataType.int16:
+        if ifm_dtype == DataType.int16 and bias_tens.dtype == DataType.int64:
+            # Reference uses reduced scaling for int16 with int64 bias
             quantised_scales = [reduced_quantise_scale(scale) for scale in scales]
         else:
             quantised_scales = [quantise_scale(scale) for scale in scales]
 
+    # Check the output quantisation to see if the scale value needs increasing to the next one
+    if first_consumer_op.get_output_quantization().next_after:
+        for i, quant_scale in enumerate(quantised_scales):
+            q_scale, q_shift = quant_scale
+            quantised_scales[i] = (q_scale + 1, q_shift)
+
     # If only 1 quantised scale is used, repeat that value for the length of the biases
     if len(quantised_scales) == 1:
         quantised_scales = [quantised_scales[0]] * len(biases)
